/***********************************************************************
 * File: monetary.h
 * 	(Extracted from
 *	   "$Id: monetary.h,v 1.5 2006/01/29 02:29:42 maoyam Exp maoyam $")
 *
 * Abstracts:
 *  This C header file defines the `monetary()' API prototyping.
 *
 * Copyright (C) 2005 Maoyam Tokyo, Japan (mailto:maoyam@mail.goo.ne.jp)
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *
 **********************************************************************/

#ifndef _MONETARY_H_ /* { */
#define _MONETARY_H_

#include <unistd.h>
#include <stddef.h>
#include <stdlib.h>
#include <stdio.h>
#include <ctype.h>
#include <string.h>
#include <memory.h>
#include <math.h>
#include <errno.h>

extern int snprintf(char *str, size_t size, const char *format, ...);

/*
 * This API converts the double value to a monetary string.
 *
 * arguments:
 *	amounts indicates the double value.
 *	max_len indicates the width of outputs.
 *	fractional_digits indicates the digits of decimal points.
 *	additions indicates the width of right hand of decimal points.
 *	str is pointing to formatted monetary string (output).
 *	sz indicates the size of str.
 *
 * returns:
 *	not NULL : the formatted monetary string (same as str).
 *	NULL : illegal arguments
 */
extern char	*monetary(double amounts
			, size_t max_len
			, size_t fractional_digits
			, size_t additions
			, char *str
			, size_t sz
			);


/*
 * This API converts the numeric string to a monetary string.
 *
 * arguments:
 *	src is pointing to the numeric string.
 *	src_sz indicates the length of string that src is pointing to.
 *	max_len indicates the width of outputs.
 *	fractional_digits indicates the digits of decimal points.
 *	additions indicates the width of right hand of decimal points.
 *	str is pointing to formatted monetary string (output).
 *	sz indicates the size of str.
 *
 * returns:
 *	not NULL : the formatted monetary string (same as str).
 *	NULL : illegal arguments
 */
extern char	*strmonetary(char	*src
			, size_t src_sz
			, size_t max_len
			, size_t fractional_digits
			, size_t additions
			, char *str
			, size_t sz
			);


/*
 * This API rounds the double typed value.
 *
 * arguments:
 *	value indicates the double typed value.
 *	scale indicates the rounding digits.  if it's from zero to 17,
 *	this rounds fractional part.  if it's from -1 to -16, this
 *	rounds integer parts.
 *	how indicates EVEN, UP or DOWN for rounding value.
 *
 * returns:
 *	INF : occurred several errors. errno indicates details.
 *		EINVAL : illegal arguments
 *	otherwise : rounded double value
 */
typedef enum round_double_how_t {
	  Round_double_EVEN	= 0
	, Round_double_UP
	, Round_double_DOWN
} Round_double_how_t;

double round_double(double value, int scale, Round_double_how_t how);


#endif /* } !defined(_MONETARY_H_) */
